@@ -48,11 +48,13 @@ following:
 
      datafile.ext can be any of the supported data file formats.  This
      includes VTK file formats (*.vtk, *.xml, *.vt[i,p,r,s,u],
-     *.pvt[i,p,r,s,u]), VRML2 (*.wrl), 3D Studio (*.3ds), PLOT3D (*.xyz)
-     and various others that are supported.  datafile.ext can also be a
-     source object not associated with a file, for example
-     ParametricSurface or PointLoad will load the corresponding data
-     sources into Mayavi.
+     *.pvt[i,p,r,s,u]), VRML2 (*.wrl), 3D Studio (*.3ds), PLOT3D
+     (*.xyz), STL, BYU, RAW, PLY,  PDB,  SLC,  FACET,  OBJ,  AVSUCD
+     (*.inp),  GAMBIT (*.neu),  Exodus  (*.exii),  PNG,  JPEG,  BMP,
+     PNM, DCM, DEM, MHA, MHD, MINC, XIMG, TIFF, and various others that
+     are supported.  datafile.ext can also be a source object not
+     associated with a file, for example ParametricSurface or PointLoad
+     will load the corresponding data sources into Mayavi.
 
 --filter filter-name
 -f filter-name
@@ -141,6 +143,24 @@ following:
      Loads a previously saved Mayavi2 visualization file passed as the
      argument.
 
+-t
+--test
+    
+     Runs the mayavi2 test suite and exits.  If run as such, this runs
+     both the TVTK and Mayavi2 unittests.  If any additional arguments
+     are passed they are passed along to the test runner.  So this may
+     be used to run other tests as well.  For example::
+
+        mayavi2 -t enthought.persistence
+
+     This will run just the tests inside the enthought.persistence
+     package.  You can also specify a directory with test files to run
+     with this, for example::
+
+        mayavi2 -t relative_path_to/integrationtests/mayavi
+
+     will run the integration tests from the mayavi sources.
+
 -v
 --verbose
 
@@ -397,6 +417,13 @@ def run_script(mayavi, script_name):
 
     return error
 
+# This runs the runtests script and sends any args to it.
+if ('-t' in sys.argv[1:]) or ('--test' in sys.argv[1:]):
+    from enthought.mayavi.tests import runtests
+    for arg in ('-t', '--test'):
+        if arg in sys.argv[1:]:
+            sys.argv.remove(arg)
+    runtests.main()
 
 # If the user just wants help messages.  Print them before importing
 # any of the big modules.
@@ -435,7 +462,10 @@ except ImportError, m:
 try:
     from enthought.mayavi.plugins.app import Mayavi, setup_logger
 except ImportError, m:
-    msg = '''%s\n%s\nCould not load envisage. Do you have the EnvisageCore and EnvisagePlugins installed?
+    msg = '''%s
+%s
+Could not load envisage. You might have a missing dependency.
+Do you have the EnvisageCore and EnvisagePlugins installed?
 
 If you installed Mayavi with easy_install, try 'easy_install <pkg_name>'.
 'easy_install Mayavi[app]' will also work.
@@ -443,9 +473,10 @@ If you installed Mayavi with easy_install, try 'easy_install <pkg_name>'.
 If you performed a source checkout and installed via 'python setup.py develop',
 be sure to run the same command in the EnvisageCore and EnvisagePlugins folders.
 
-If these packages appear to be installed, check your numpy installation. If you
-need numpy, 'easy_install numpy' will install numpy.
-http://numpy.scipy.org
+If these packages appear to be installed, check that your numpy and
+configobj are installed and working. If you need numpy, 'easy_install numpy' 
+will install numpy. Similarly, 'easy_install configobj' will install 
+configobj.
         ''' % (m, '_'*80)
     raise ImportError(msg)
 
