@@ -11,7 +11,7 @@ pipeline in a procedural way.
 # Enthought library imports.
 import enthought.mayavi.modules.api as modules
 from enthought.traits.api import String, CFloat, Instance, HasTraits, \
-            Trait, CArray, true, Any
+            Trait, CArray, true, Any, Range
 import tools
 from figure import draw, gcf
 
@@ -322,6 +322,9 @@ class Axes(AxesLikeModuleFactory):
     zlabel = String(None, adapts='axes.z_label',
                 help='the label of the z axis')
 
+    nb_labels = Range(0, 50, 2, adapts='axes.number_of_labels',
+                desc='The number of labels along each direction') 
+
     ranges = Trait(None, None, CArray(shape=(6,)),
                     help="""[xmin, xmax, ymin, ymax, zmin, zmax]
                             Ranges of the labels displayed on the axes.
@@ -348,7 +351,6 @@ class Axes(AxesLikeModuleFactory):
         if self.ranges is None:
             axes.axes.ranges = \
                 axes.module_manager.source.outputs[0].bounds
-            axes.axes.use_ranges = True
 
     def _ranges_changed(self):
         if self.ranges is not None:
@@ -411,7 +413,7 @@ class OrientationAxesFactory(SingletonModuleFactory):
 
     _target = Instance(modules.OrientationAxes, ())
 
-orientationaxes = make_function(OrientationAxesFactory)
+orientation_axes = make_function(OrientationAxesFactory)
 
 
 ###############################################################################
@@ -422,8 +424,9 @@ class Text(ModuleFactory):
         
             text(x, y, text, ...) 
 
-        x, and y are the position of the origin of the text on the 2D 
-        projection of the figure. If a z keyword argument is given, the
+        x, and y are the position of the origin of the text. If no z
+        keyword argument is given, x and y are the 2D projection of the 
+        figure, they belong to [0, 1]. If a z keyword  argument is given, the 
         text is positionned in 3D, in figure coordinnates.
         """
 
@@ -436,11 +439,17 @@ class Text(ModuleFactory):
 
     _target = Instance(modules.Text, ())
 
+    opacity = CFloat(1, adapts="property.opacity",
+                        help="""The opacity of the text.""")
+
     def __init__(self, x, y, text, **kwargs):
         """ Override init as for different positional arguments."""
         if 'z' in kwargs and kwargs['z'] is not None:
             self._target.z_position = kwargs['z']
             self._target.position_in_3d = True
+        elif not (x<1. and x>0. and y>0. and y<1.):
+            raise ValueError('Text positions should be in [0, 1] if no z'
+                'position is given')
         super(Text, self).__init__(None, **kwargs)
         self._target.text       = text
         self._target.x_position = x
@@ -489,6 +498,11 @@ class Title(SingletonModuleFactory):
             self.name = kwargs['name'] = 'Title'
         super(Title, self).__init__(**kwargs)
         self._target.text = self._text
+        # We need to set position after Text is initiated, as text will 
+        # override these positions
+        self._target.y_position = self.height
+        self._size_changed()
+
 
 title = make_function(Title)
 
