@@ -5,7 +5,7 @@ operation. They should always return the module object created, for
 consistency, and because retrieving the data_source from a module object
 is possible via tools._find_data
 
-Each helper function should have a test function assoiciated with it,
+Each helper function should have a test function associated with it,
 both for testing and to ilustrate its use.
 """
 
@@ -15,7 +15,7 @@ both for testing and to ilustrate its use.
 
 from modules import VectorsFactory, StreamlineFactory, GlyphFactory, \
             IsoSurfaceFactory, SurfaceFactory, ContourSurfaceFactory, \
-            glyph_mode_dict
+            ImageActorFactory, glyph_mode_dict
 from sources import vector_scatter, vector_field, scalar_scatter, \
             scalar_field, line_source, array2d_source, grid_source, \
             triangular_mesh_source, vertical_vectors_source
@@ -95,11 +95,6 @@ class Pipeline(HasTraits):
                 if key in keywords:
                     this_kwargs[key] = value
             object = pipe(object, **this_kwargs)._target
-        # Inject the magical mlab source trait.
-        if hasattr(self.source, 'mlab_source'):
-            ms = self.source.mlab_source
-            object.add_trait('mlab_source', Instance(ms.__class__))
-            object.mlab_source = ms
         return object
 
     def get_all_traits(self):
@@ -115,11 +110,109 @@ class Pipeline(HasTraits):
 
 
 ############################################################################# 
+class Points3d(Pipeline):
+    """ 
+    Plots glyphs (like points) at the position of the supplied data.
 
-class Quiver3D(Pipeline):
+    **Function signatures**::
+
+        points3d(x, y, z...)
+        points3d(x, y, z, s, ...)
+        points3d(x, y, z, f, ...)
+
+    x, y and z are numpy arrays, or lists, all of the same shape, giving 
+    the positions of the points.
+
+    If only 3 arrays x, y, z are given, all the points are drawn with the 
+    same size and color. 
+    
+    In addition, you can pass a fourth array s of the same
+    shape as x, y, and z giving an associated scalar value for each
+    point, or a function f(x, y, z) returning the scalar value. This
+    scalar value can be used to modulate the color and the size of the
+    points."""
+
+    _source_function = Callable(scalar_scatter)
+
+    _pipeline = [GlyphFactory, ]
+
+    scale_factor = Any('auto', help='The scaling applied to the glyphs. '
+                        'the simple of the glyph is by default calculated '
+                        'from the inter-glyph spacing. Specify a float to '
+                        'give the maximum glyph size in drawing units'
+                        )
+    
+    def __call__(self, *args, **kwargs):
+        """ Override the call to be able to scale automaticaly the glyphs.
+        """
+        scale_factor =  kwargs.get('scale_factor', 'auto')
+        if scale_factor == 'auto':
+            kwargs['scale_factor'] = 1
+        g = Pipeline.__call__(self, *args, **kwargs)
+        if scale_factor == 'auto':
+            g.glyph.glyph.scale_factor = \
+                             tools._typical_distance(g.mlab_source.dataset)
+            g.glyph.glyph.clamping = True
+        else:
+            g.glyph.glyph.clamping = False
+        return g
+
+points3d = document_pipeline(Points3d())
+
+
+def test_points3d():
+    t = numpy.linspace(0, 4*numpy.pi, 20)
+    cos = numpy.cos
+    sin = numpy.sin
+
+    x = sin(2*t)
+    y = cos(t)
+    z = cos(2*t)
+    s = 2+sin(t)
+
+    return points3d(x, y, z, s, colormap="copper", scale_factor=.25)
+
+def test_points3d_anim():
+    """Animates the test_points3d example."""
+    g = test_points3d()
+    t = numpy.linspace(0, 4*numpy.pi, 20)
+    # Animate the points3d.
+    ms = g.mlab_source
+    for i in range(10):
+        ms.z = numpy.cos(2*t*0.1*(i+1))
+    return g
+
+
+def test_molecule():
+    """Generates and shows a Caffeine molecule."""
+    o = [[30, 62, 19],[8, 21, 10]]
+    ox, oy, oz = map(numpy.array, zip(*o))
+    n = [[31, 21, 11], [18, 42, 14], [55, 46, 17], [56, 25, 13]]
+    nx, ny, nz = map(numpy.array, zip(*n))
+    c = [[5, 49, 15], [30, 50, 16], [42, 42, 15], [43, 29, 13], [18, 28, 12],
+         [32, 6, 8], [63, 36, 15], [59, 60, 20]]
+    cx, cy, cz = map(numpy.array, zip(*c))
+    h = [[23, 5, 7], [32, 0, 16], [37, 5, 0], [73, 36, 16], [69, 60, 20],
+         [54, 62, 28], [57, 66, 12], [6, 59, 16], [1, 44, 22], [0, 49, 6]]
+    hx, hy, hz = map(numpy.array, zip(*h))
+
+    oxygen = points3d(ox, oy, oz, scale_factor=16, scale_mode='none',
+                                resolution=20, color=(1,0,0), name='Oxygen')
+    nitrogen = points3d(nx, ny, nz, scale_factor=20, scale_mode='none', 
+                                resolution=20, color=(0,0,1), name='Nitrogen')
+    carbon = points3d(cx, cy, cz, scale_factor=20, scale_mode='none', 
+                                resolution=20, color=(0,1,0), name='Carbon')
+    hydrogen = points3d(hx, hy, hz, scale_factor=10, scale_mode='none', 
+                                resolution=20, color=(1,1,1), name='Hydrogen')
+
+    return oxygen, nitrogen, carbon, hydrogen
+
+############################################################################# 
+
+class Quiver3D(Points3d):
     """
     Plots glyphs (like arrows) indicating the direction of the vectors
-    for a 3D volume of data supplied as arguments.
+    at the positions supplied.
 
     **Function signatures**::
     
@@ -127,11 +220,19 @@ class Quiver3D(Pipeline):
         quiver3d(x, y, z, u, v, w, ...)
         quiver3d(x, y, z, f, ...)
     
-    If only 3 arrays u, v, w are passed the x, y and z arrays are assumed to be
-    made from the indices of vectors.
-    
-    If 4 positional arguments are passed the last one must be a callable, f, 
-    that returns vectors. """
+    u, v, w are numpy arrays giving the components of the vectors.
+
+    If only 3 arrays, u, v, and w are passed, they must be 3D arrays, and 
+    the positions of the arrows are assumed to be the indices of the
+    corresponding points in the (u, v, w) arrays.
+
+    If 6 arrays, (x, y, z, u, v, w) are passed, the 3 first arrays give
+    the position of the arrows, and the 3 last the components. They
+    can be of any shape.
+
+    If 4 positional arguments, (x, y, z, f) are passed, the last one must be 
+    a callable, f, that returns vectors components (u, v, w) given the
+    positions (x, y, z)."""
 
     scalars = Array(help="""optional scalar data.""")
 
@@ -142,13 +243,20 @@ class Quiver3D(Pipeline):
 
 quiver3d = document_pipeline(Quiver3D())
 
-
 def test_quiver3d():
+    x, y, z = numpy.mgrid[-2:3, -2:3, -2:3]
+    r = numpy.sqrt(x**2 + y**2 + z**4)
+    u = y*numpy.sin(r)/(r+0.001)
+    v = -x*numpy.sin(r)/(r+0.001)
+    w = numpy.zeros_like(z)
+    obj = quiver3d(x, y, z, u, v, w, line_width=3, scale_factor=1)
+    return obj
+
+
+def test_quiver3d_cone():
     dims = [8, 8, 8]
     xmin, xmax, ymin, ymax, zmin, zmax = [-5,5,-5,5,-5,5]
-    x, y, z = numpy.mgrid[xmin:xmax:dims[0]*1j,
-                          ymin:ymax:dims[1]*1j,
-                          zmin:zmax:dims[2]*1j]
+    x, y, z = numpy.mgrid[-5:5:8j, -5:5:8j, -5:5:8j]
     x = x.astype('f')
     y = y.astype('f')
     z = z.astype('f')
@@ -185,7 +293,7 @@ def test_quiver3d_2d_data():
 ############################################################################# 
 class Flow(Pipeline):
     """
-    Creates streamlines following the flow of a vector field. 
+    Creates trajectories of particles following the flow of a vector field. 
                       
     **Function signatures**::
     
@@ -193,15 +301,24 @@ class Flow(Pipeline):
         flow(x, y, z, u, v, w, ...)
         flow(x, y, z, f, ...)
 
-    If only 3 arrays u, v, w are passed the x, y and z arrays are assumed to be
-    made from the indices of vectors.
+    u, v, w are numpy arrays giving the components of the vectors.
 
-    If the x, y and z arrays are passed they are supposed to have been
-    generated by `numpy.mgrid`. The function builds a scalar field assuming 
-    the points are regularily spaced.
+    If only 3 arrays, u, v, and w are passed, they must be 3D arrays, and 
+    the positions of the arrows are assumed to be the indices of the
+    corresponding points in the (u, v, w) arrays.
 
-    If 4 positional arguments are passed the last one must be a callable, f, 
-    that returns vectors."""
+    If 6 arrays, (x, y, z, u, v, w) are passed, the 3 first arrays give
+    the position of the arrows, and the 3 last the components. The x, y
+    and z arrays are then supposed to have been generated by
+    `numpy.mgrid`, in other words, they are 3D arrays, with positions
+    lying on a 3D orthogonal and regularily spaced grid with nearest
+    neighboor in space matching nearest neighboor in the array. The
+    function builds a vector field assuming  the points are regularily
+    spaced.
+
+    If 4 positional arguments, (x, y, z, f) are passed, the last one must be 
+    a callable, f, that returns vectors components (u, v, w) given the
+    positions (x, y, z)."""
 
     scalars = Array(help="""optional scalar data.""")
 
@@ -222,10 +339,19 @@ class Flow(Pipeline):
             self.pipeline.pop(0)
         return self.build_pipeline()
 
-
 flow = document_pipeline(Flow())
 
 def test_flow():
+    x, y, z = numpy.mgrid[0:5, 0:5, 0:5]
+    r = numpy.sqrt(x**2 + y**2 + z**4)
+    u = y*numpy.sin(r)/r
+    v = -x*numpy.sin(r)/r
+    w = numpy.zeros_like(z)
+    obj = flow(u, v, w)
+    return obj
+
+
+def test_flow_tubes():
     dims = [32, 32, 32]
     xmin, xmax, ymin, ymax, zmin, zmax = [-5,5,-5,5,-5,5]
     x, y, z = numpy.mgrid[xmin:xmax:dims[0]*1j,
@@ -294,82 +420,6 @@ def test_flow_scalars():
     return obj
 
 
-############################################################################# 
-class Points3d(Pipeline):
-    """ 
-    Plots glyphs (like points) at the position of the supplied data.
-
-    **Function signatures**::
-
-        points3d(scalardata, ...)
-        points3d(x, y, z...)
-        points3d(x, y, z, s, ...)
-        points3d(x, y, z, f, ...)
-
-    If only one positional argument is passed, it should be VTK data
-    object with scalar data.
-
-    If only 3 arrays x, y, z all the points are drawn with the same size
-    and color
-
-    If 4 positional arguments are passed the last one can be an array s
-    or a callable f that gives the size and color of the glyph."""
-
-    _source_function = Callable(scalar_scatter)
-
-    _pipeline = [GlyphFactory, ]
-
-
-points3d = document_pipeline(Points3d())
-
-
-def test_points3d():
-    t = numpy.linspace(0, 4*numpy.pi, 20)
-    cos = numpy.cos
-    sin = numpy.sin
-
-    x = sin(2*t)
-    y = cos(t)
-    z = cos(2*t)
-    s = 2+sin(t)
-
-    return points3d(x, y, z, s, colormap="copper", scale_factor=.25)
-
-def test_points3d_anim():
-    """Animates the test_points3d example."""
-    g = test_points3d()
-    t = numpy.linspace(0, 4*numpy.pi, 20)
-    # Animate the points3d.
-    ms = g.mlab_source
-    for i in range(10):
-        ms.z = numpy.cos(2*t*0.1*(i+1))
-    return g
-
-
-def test_molecule():
-    """Generates and shows a Caffeine molecule."""
-    o = [[30, 62, 19],[8, 21, 10]]
-    ox, oy, oz = map(numpy.array, zip(*o))
-    n = [[31, 21, 11], [18, 42, 14], [55, 46, 17], [56, 25, 13]]
-    nx, ny, nz = map(numpy.array, zip(*n))
-    c = [[5, 49, 15], [30, 50, 16], [42, 42, 15], [43, 29, 13], [18, 28, 12],
-         [32, 6, 8], [63, 36, 15], [59, 60, 20]]
-    cx, cy, cz = map(numpy.array, zip(*c))
-    h = [[23, 5, 7], [32, 0, 16], [37, 5, 0], [73, 36, 16], [69, 60, 20],
-         [54, 62, 28], [57, 66, 12], [6, 59, 16], [1, 44, 22], [0, 49, 6]]
-    hx, hy, hz = map(numpy.array, zip(*h))
-
-    oxygen = points3d(ox, oy, oz, scale_factor=16, scale_mode='none',
-                                resolution=20, color=(1,0,0), name='Oxygen')
-    nitrogen = points3d(nx, ny, nz, scale_factor=20, scale_mode='none', 
-                                resolution=20, color=(0,0,1), name='Nitrogen')
-    carbon = points3d(cx, cy, cz, scale_factor=20, scale_mode='none', 
-                                resolution=20, color=(0,1,0), name='Carbon')
-    hydrogen = points3d(hx, hy, hz, scale_factor=10, scale_mode='none', 
-                                resolution=20, color=(1,1,1), name='Hydrogen')
-
-    return oxygen, nitrogen, carbon, hydrogen
-
 ############################################################################# 
 class Contour3d(Pipeline):
     """
@@ -378,7 +428,22 @@ class Contour3d(Pipeline):
     **Function signatures**::
 
         contour3d(scalars, ...)
-        contour3d(scalarfield, ...)"""
+        contour3d(x, y, z, scalars, ...)
+
+    scalars is a 3D numpy arrays giving the data on a grid.
+
+    If 4 arrays, (x, y, z, scalars) are passed, the 3 first arrays give
+    the position of the arrows, and the last the scalar value. The x, y
+    and z arrays are then supposed to have been generated by
+    `numpy.mgrid`, in other words, they are 3D arrays, with positions
+    lying on a 3D orthogonal and regularily spaced grid with nearest
+    neighboor in space matching nearest neighboor in the array. The
+    function builds a scalar field assuming  the points are regularily
+    spaced.
+
+    If 4 positional arguments, (x, y, z, f) are passed, the last one
+    can also be a callable, f, that returns vectors components (u, v, w) 
+    given the positions (x, y, z)."""
 
     _source_function = Callable(scalar_field)
 
@@ -389,16 +454,8 @@ contour3d = document_pipeline(Contour3d())
 
 
 def test_contour3d():
-    dims = [64, 64, 64]
-    xmin, xmax, ymin, ymax, zmin, zmax = [-5,5,-5,5,-5,5]
-    x, y, z = numpy.ogrid[xmin:xmax:dims[0]*1j,
-                          ymin:ymax:dims[1]*1j,
-                          zmin:zmax:dims[2]*1j]
-    x = x.astype('f')
-    y = y.astype('f')
-    z = z.astype('f')
-
-    sin = numpy.sin
+    x, y, z = numpy.ogrid[-5:5:64j, -5:5:64j, -5:5:64j]
+    
     scalars = x*x*0.5 + y*y + z*z*2.0
 
     obj = contour3d(scalars, contours=4, transparent=True)
@@ -433,7 +490,11 @@ class Plot3d(Pipeline):
     **Function signatures**::
 
         plot3d(x, y, z, ...)
-        plot3d(x, y, z, s, ...)"""
+        plot3d(x, y, z, s, ...)
+        
+    x, y, z and s are numpy arrays or lists of the same shape. x, y and z
+    give the positions of the successive points of the line. s is an
+    optional scalar value associated with each point."""
 
     tube_radius = Trait(0.025, CFloat, None,
                         adapts='filter.radius',
@@ -467,7 +528,7 @@ def test_plot3d():
     n_mer, n_long = 6, 11
     pi = numpy.pi
     dphi = pi/1000.0 
-    phi = numpy.arange(0.0, 2*pi + 0.5*dphi, dphi, 'd')
+    phi = numpy.arange(0.0, 2*pi + 0.5*dphi, dphi)
     mu = phi*n_mer
     x = numpy.cos(mu)*(1+numpy.cos(n_long*mu/n_mer)*0.5)
     y = numpy.sin(mu)*(1+numpy.cos(n_long*mu/n_mer)*0.5)
@@ -501,16 +562,18 @@ def test_plot3d_anim():
 ############################################################################# 
 class ImShow(Pipeline):
     """
-    Allows one to view a 2D Numeric array as an image.  This works
-    best for very large arrays (like 1024x1024 arrays).
+    View a 2D array as an image. 
 
     **Function signatures**::
 
-        imshow(2darray, ...)"""
+        imshow(s, ...)
+    
+    s is a 2 dimension array. The values of s are mapped to a color using
+    the colormap."""
 
     _source_function = Callable(array2d_source)
 
-    _pipeline = [SurfaceFactory, ]
+    _pipeline = [ImageActorFactory, ]
 
 
 imshow = document_pipeline(ImShow())
@@ -526,7 +589,7 @@ def test_imshow():
 ############################################################################# 
 class Surf(Pipeline):
     """
-    Plots a surface using regularly spaced elevation data supplied as a 2D 
+    Plots a surface using regularly-spaced elevation data supplied as a 2D 
     array.
 
     **Function signatures**::
@@ -535,11 +598,7 @@ class Surf(Pipeline):
         surf(x, y, s, ...)
         surf(x, y, f, ...)        
     
-    If 3 positional arguments are passed the last one must be an array s,
-    or a callable, f, that returns an array. x and y give the
-    coordinnates of positions corresponding to the s values. 
-    
-    z is the elevation matrix.
+    s is the elevation matrix, a 2D array.
     
     x and y can be 1D or 2D arrays (such as returned by numpy.ogrid or
     numpy.mgrid), but the points should be located on an orthogonal grid
@@ -548,9 +607,13 @@ class Surf(Pipeline):
     arbitrary-shaped position arrays (non-orthogonal grids), see the mesh 
     function.
 
-    If only 1 array s is passed the x and y arrays are assumed to be
+    If only 1 array s is passed, the x and y arrays are assumed to be
     made from the indices of arrays, and an uniformly-spaced data set is 
-    created."""
+    created.
+    
+    If 3 positional arguments are passed the last one must be an array s,
+    or a callable, f, that returns an array. x and y give the
+    coordinnates of positions corresponding to the s values."""
 
     _source_function = Callable(array2d_source)
 
@@ -676,13 +739,12 @@ class Mesh(Pipeline):
 
         mesh(x, y, z, ...)
     
-    x, y, z are 2D arrays giving the positions of the vertices of the surface.
-    The connectivity between these points is implied by the connectivity on 
-    the arrays. 
+    x, y, z are 2D arrays, all of the same shape, giving the positions of
+    the vertices of the surface. The connectivity between these points is
+    implied by the connectivity on the arrays. 
 
     For simple structures (such as orthogonal grids) prefer the surf function,
-    as it will create more efficient data structures.
-    """
+    as it will create more efficient data structures."""
 
     scale_mode = Trait('none', {'none':'data_scaling_off',
                                 'scalar':'scale_by_scalar',
@@ -806,8 +868,8 @@ def test_fancy_mesh():
 ############################################################################# 
 class ContourSurf(Pipeline):
     """
-    Plots a the contours of asurface using grid spaced data supplied as 2D 
-    arrays.
+    Plots a the contours of a surface using grid-spaced data for
+    elevation supplied as a 2D array.
 
     **Function signatures**::
 
@@ -815,9 +877,24 @@ class ContourSurf(Pipeline):
         contour_surf(x, y, s, ...)
         contour_surf(x, y, f, ...)        
 
-    If only one array s is passed the x and y arrays are assumed to be made 
-    of the indices of s.
-    s is the elevation matrix."""
+    s is the elevation matrix, a 2D array. The contour lines plotted
+    are lines of equal s value.
+    
+    x and y can be 1D or 2D arrays (such as returned by numpy.ogrid or
+    numpy.mgrid), but the points should be located on an orthogonal grid
+    (possibly non-uniform). In other words, all the points sharing a same
+    index in the s array need to have the same x or y value. For 
+    arbitrary-shaped position arrays (non-orthogonal grids), see the mesh 
+    function.
+
+    If only 1 array s is passed, the x and y arrays are assumed to be
+    made from the indices of arrays, and an uniformly-spaced data set is 
+    created.
+    
+    If 3 positional arguments are passed the last one must be an array s,
+    or a callable, f, that returns an array. x and y give the
+    coordinnates of positions corresponding to the s values."""
+
 
     _source_function = Callable(array2d_source)
 
@@ -838,7 +915,8 @@ def test_contour_surf():
     return s
 
 ############################################################################# 
-# Expose only the glyphs that make (more or less) sens for a barchart.
+
+# Expose only the glyphs that make (more or less) sense for a barchart.
 bar_mode_dict = dict()
 for item in ('cube', '2dtriangle', '2dsquare', '2dvertex', '2dthick_cross', 
              '2ddiamond', '2dcross', '2dcircle'):
@@ -850,7 +928,7 @@ class BarChart(Pipeline):
     histogram-like plots.
 
     This functions accepts a wide variety of inputs, with positions given
-    in 2D or in 3D.
+    in 2-D or in 3-D.
 
     **Function signatures**::
 
@@ -860,7 +938,7 @@ class BarChart(Pipeline):
         barchart(x, y, z, s, ...)
         barchart(x, y, z, f, ...)
 
-    If only one positional argument is passed, it can be a 1D, 2D, or 3D
+    If only one positional argument is passed, it can be a 1-D, 2-D, or 3-D
     array giving the length of the vectors. The positions of the data
     points are deducted from the indices of array, and an
     uniformly-spaced data set is created.
@@ -895,7 +973,7 @@ class BarChart(Pipeline):
             gs.glyph_source = gs.glyph_list[-1]
         # Position the glyph tail on the point.
         gs.glyph_position = 'tail'
-        gs.glyph_source.center = (0.5, 0.0, 0.5)
+        gs.glyph_source.center = (0.0, 0.0, 0.5)
         g.glyph.glyph.orient = False
         if not 'color' in kwargs:
             g.glyph.color_mode = 'color_by_scalar'
@@ -933,7 +1011,7 @@ class TriangularMesh(Mesh):
 
     **Function signatures**::
 
-        mesh(x, y, z, triangles ...)
+        triangular_mesh(x, y, z, triangles ...)
     
     x, y, z are arrays giving the positions of the vertices of the surface.
     triangles is a list of triplets (or an array) list the vertices in
@@ -954,7 +1032,7 @@ def test_triangular_mesh():
         triangles.
     """
     n = 8
-    t = numpy.linspace(0, 2*numpy.pi, n)
+    t = numpy.linspace(-numpy.pi, numpy.pi, n)
     z = numpy.exp(1j*t)
     x = z.real.copy()
     y = z.imag.copy()
