@@ -763,6 +763,11 @@ class MTriangularMeshSource(MlabSource):
         self.update()
 
     def _triangles_changed(self, triangles):
+        if triangles.min() < 0:
+            raise ValueError, 'The triangles array has negative values'
+        if triangles.max() > self.x.size:
+            raise ValueError, 'The triangles array has values larger than' \
+                                        'the number of points'
         self.dataset.polys = triangles
         self.update() 
    
@@ -777,8 +782,14 @@ def convert_to_arrays(args):
     """
     args = list(args)
     for index, arg in enumerate(args):
-        if not hasattr(arg, 'shape') and not callable(arg):
-            args[index] = numpy.atleast_1d(numpy.array(arg))
+        if not callable(arg):
+            if not hasattr(arg, 'shape'):
+                arg = numpy.atleast_1d(numpy.array(arg))
+            if numpy.any(numpy.isinf(arg)):
+                raise ValueError("""Input array contains infinite values
+                You can remove them using: a[numpy.isinf(a)] = numpy.nan
+                """)
+            args[index] = arg
     return args
 
 def process_regular_vectors(*args):
@@ -886,7 +897,9 @@ def vector_scatter(*args, **kwargs):
                  If None, the source is not added to any figure, and will 
                  be added automatically by the modules or
                  filters. If False, no figure will be created by modules
-                 or filters applied to the source."""
+                 or filters applied to the source: the source can only
+                 be used for testing, or numerical algorithms, not
+                 visualization."""
     x, y, z, u, v, w = process_regular_vectors(*args)
 
     scalars = kwargs.pop('scalars', None)
@@ -931,7 +944,9 @@ def vector_field(*args, **kwargs):
                  If None, the source is not added to any figure, and will 
                  be added automatically by the modules or
                  filters. If False, no figure will be created by modules
-                 or filters applied to the source."""
+                 or filters applied to the source: the source can only
+                 be used for testing, or numerical algorithms, not
+                 visualization."""
     if len(args) == 3:
         x = y = z = numpy.atleast_3d(1)
         u, v, w = [numpy.atleast_3d(a) for a in args]
@@ -971,7 +986,9 @@ def scalar_scatter(*args, **kwargs):
                  If None, the source is not added to any figure, and will 
                  be added automatically by the modules or
                  filters. If False, no figure will be created by modules
-                 or filters applied to the source."""
+                 or filters applied to the source: the source can only
+                 be used for testing, or numerical algorithms, not
+                 visualization."""
     x, y, z, s = process_regular_scalars(*args)
 
     if s is not None:
@@ -1014,7 +1031,9 @@ def scalar_field(*args, **kwargs):
                  If None, the source is not added to any figure, and will 
                  be added automatically by the modules or
                  filters. If False, no figure will be created by modules
-                 or filters applied to the source."""
+                 or filters applied to the source: the source can only
+                 be used for testing, or numerical algorithms, not
+                 visualization."""
     if len(args) == 1:
         # Be lazy, don't create three big arrays for 1 input array. The
         # MArraySource is clever-enough to handle flat arrays
@@ -1051,7 +1070,9 @@ def line_source(*args, **kwargs):
                  If None, the source is not added to any figure, and will 
                  be added automatically by the modules or
                  filters. If False, no figure will be created by modules
-                 or filters applied to the source."""
+                 or filters applied to the source: the source can only
+                 be used for testing, or numerical algorithms, not
+                 visualization."""
     if len(args)==1:
         raise ValueError, "wrong number of arguments"    
     x, y, z, s = process_regular_scalars(*args)
@@ -1096,7 +1117,9 @@ def array2d_source(*args, **kwargs):
                  If None, the source is not added to any figure, and will 
                  be added automatically by the modules or
                  filters. If False, no figure will be created by modules
-                 or filters applied to the source.
+                 or filters applied to the source: the source can only
+                 be used for testing, or numerical algorithms, not
+                 visualization.
         
         :mask: Mask points specified in a boolean masking array.
     """
@@ -1135,12 +1158,15 @@ def grid_source(x, y, z, **kwargs):
                  If None, the source is not added to any figure, and will 
                  be added automatically by the modules or
                  filters. If False, no figure will be created by modules
-                 or filters applied to the source.
+                 or filters applied to the source: the source can only
+                 be used for testing, or numerical algorithms, not
+                 visualization.
         """
     scalars = kwargs.pop('scalars', None)
     if scalars is None:
         scalars = z
 
+    x, y, z, scalars = convert_to_arrays((x, y, z, scalars))
     data_source = MGridSource()
     data_source.reset(x=x, y=y, z=z, scalars=scalars)
 
@@ -1185,7 +1211,9 @@ def vertical_vectors_source(*args, **kwargs):
                  If None, the source is not added to any figure, and will 
                  be added automatically by the modules or
                  filters. If False, no figure will be created by modules
-                 or filters applied to the source.
+                 or filters applied to the source: the source can only
+                 be used for testing, or numerical algorithms, not
+                 visualization.
     """
     if len(args) == 3:
         x, y, data = args
@@ -1224,10 +1252,17 @@ def triangular_mesh_source(x, y, z, triangles, **kwargs):
                  If None, the source is not added to any figure, and will 
                  be added automatically by the modules or
                  filters. If False, no figure will be created by modules
-                 or filters applied to the source.
+                 or filters applied to the source: the source can only
+                 be used for testing, or numerical algorithms, not
+                 visualization.
         """
     x, y, z, triangles = convert_to_arrays((x, y, z, triangles))
 
+    if triangles.min() < 0:
+        raise ValueError, 'The triangles array has negative values'
+    if triangles.max() > x.size:
+        raise ValueError, 'The triangles array has values larger than' \
+                                    'the number of points'
     scalars = kwargs.pop('scalars', None)
     if scalars is None:
         scalars = z
